use crate::db::Database;

use super::{utils::BotData, Error, Result};

use poise::{serenity_prelude as serenity, Event};
use tracing::{error, info};

async fn handle_everyone_mention(
    ctx: &serenity::Context,
    database: &Database,
    message: &serenity::Message,
) -> Result {
    use serenity::ChannelId;
    if let Some(guild_id) = message.guild_id {
        if message.mention_everyone {
            let author = message.author.clone();
            let message_channel = message.channel_id;
            let channels: Vec<ChannelId> = database
                .get_logging_channels(guild_id)
                .await?
                .iter()
                .map(|channel_id| serenity::ChannelId(channel_id.to_owned()))
                .collect();
            for channel in &channels {
                channel
                    .send_message(&ctx, |m| {
                        m.embed(|e| {
                            e.title("Someone mentioned everyone!")
                                .field("Author", author.clone(), true)
                                .field(
                                    "When",
                                    message.timestamp.naive_local().to_string(),
                                    true,
                                )
                                .field(
                                    "Channel",
                                    format!("<#{message_channel}>"),
                                    true,
                                )
                                .field("Link", format!("https://discord.com/channels/{guild_id}/{}/{}", channel.0, message.id), false)
                        })
                    })
                    .await
                    .map_err(|e| {
                        error!("Failed to send message: {e:?}");
                        e
                    })?;
            }
        }
    } else {
        error!("Could not determine guild id of message {message:?}");
    }
    Ok(())
}

pub async fn event_handler(
    ctx: &serenity::Context,
    event: &Event<'_>,
    _framework: poise::FrameworkContext<'_, BotData, Error>,
    data: &BotData,
) -> Result {
    match event {
        Event::Ready { data_about_bot } => {
            info!("Logged in as {}", data_about_bot.user.name);
        }
        Event::Message { new_message } => {
            handle_everyone_mention(ctx, &data.database, new_message).await?;
        }
        _ => {}
    }
    Ok(())
}
